// CalmWatchdog [-drl] reset-timeout  reset-after-reboot-timeout time-between-calls
// -d = debug
// -r = reset
// -l = loop
//
// in case of USB-WatchLight: CalmWatchdog [-drl] secondsToTurnRed secondsToTurnYellow timeBetweenCalms

#include "stdafx.h"
#include "USBaccess.h"

int 
main(int argc, char* argv[]) {
	CUSBaccess CWusb ;
	int timeout1=1, timeout2=0 ;
	int timeBetweenCalms = 1000 ;
	char *progname = *argv++ ;
	argc-- ;
	int debug = 0 ;
	int doReset = 0 ;
	int loop = 0 ;

	while (argc > 0 && argv[0][0] == '-') {
		switch (argv[0][1] | 0x20) {
			case 'd':
				debug = 1 ;
				break ;
			case 'r':
				doReset = 1 ;
				break ;
			case 'l':
				loop = 1 ;
				break ;
			}
		argc-- ;
		argv++ ;
		}

	if (argc >= 1) {
		if (sscanf(*argv, "%d", &timeout1) != 1) {
			printf("illegal timeout 1 (%s)\n", *argv) ;
			exit(1) ;
			}
		argc-- ;
		argv++ ;
		}

	if (argc >= 1) {
		if (sscanf(*argv, "%d", &timeout2) != 1) {
			printf("illegal timeout 2 (%s)\n", *argv) ;
			exit(1) ;
			}
		argc-- ;
		argv++ ;
		}

	if (argc >= 1) {
		if (sscanf(*argv, "%d", &timeBetweenCalms) != 1) {
			printf("illegal time between calms (%s)\n", *argv) ;
			exit(1) ;
			}
		argc-- ;
		argv++ ;
		}

	int USBcount = CWusb.OpenCleware() ;

	if (debug)
		printf("OpenCleware found %d devices\n", USBcount) ;



	do {
		int appropriateFound = 0 ;
		for (int devID=0 ; devID < USBcount ; devID++) {
			int type = CWusb.GetUSBType(devID) ;
			if (type == CUSBaccess::WATCHDOG_DEVICE || type == CUSBaccess::AUTORESET_DEVICE) {
				if (doReset) {
					if (debug)
						printf("first do a reset and wait 1 s\n") ;
				// first do a reset 
					CWusb.ResetDevice(devID) ;
					Sleep(1000) ;		// wait 1 second to settle 
					}
				int state = CWusb.GetSwitch(devID, CUSBaccess::SWITCH_1) ;

				if (debug)
					printf("current state = %d\n", state) ;

				CWusb.CalmWatchdog(devID, timeout1, timeout2) ;	// timeout 1 minute
				if (loop)
					Sleep(timeBetweenCalms) ;
				appropriateFound = 1 ;
				}
			if (type == CUSBaccess::WATCHDOGXP_DEVICE) {
				CWusb.CalmWatchdog(devID, timeout1, timeout2) ;	// timeout in seconds
				if (debug)
					printf("CalmWatchDog(%d,%d)\n", timeout1, timeout2) ;
				if (loop)
					Sleep(timeBetweenCalms*1000) ;
				appropriateFound = 1 ;
				}
			}

		if (!appropriateFound) {
			printf("No appropriate device found\n", USBcount) ;
			break ;
			}

		} while (loop) ;

	CWusb.CloseCleware() ;

	return 0;
	}

